<?php

namespace Drupal\commerce_registration\Plugin\Field\FieldFormatter;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\registration\Plugin\Field\FieldFormatter\RegistrationLinkFormatter as BaseRegistrationLinkFormatter;

/**
 * Extends the plugin implementation of the 'registration_link' formatter.
 *
 * Adds a required route parameter for commerce products when needed.
 */
class RegistrationLinkFormatter extends BaseRegistrationLinkFormatter {

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode): array {
    $elements = [];
    if ($entity = $items->getEntity()) {
      if (isset($items[0])) {
        if ($id = $items[0]->getValue()['registration_type']) {
          $registration_type = $this->entityTypeManager
            ->getStorage('registration_type')
            ->load($id);
          if ($registration_type) {
            // Check access.
            $access_result = $this->entityTypeManager
              ->getAccessControlHandler('registration')
              ->createAccess($id, NULL, [], TRUE);
            if ($access_result->isAllowed()) {
              /** @var \Drupal\registration\HostEntityInterface $host_entity */
              $host_entity = $this->entityTypeManager
                ->getHandler($entity->getEntityTypeId(), 'registration_host_entity')
                ->createHostEntity($entity, $langcode);

              // Only show the link if the host entity is open for registration.
              // For performance reasons, the isAvailableForRegistration
              // method, which takes capacity into account, is not used here.
              // That method has a dependency on when registrations are added,
              // updated or deleted, causing this render array to be rebuilt
              // more often than desired. The minor downside is the possibility
              // that capacity has been reached - if that happens, the user may
              // click on a link that takes them to a page indicating there is
              // no more room, which is a minor UX consideration.
              $validation_result = $host_entity->isOpenForRegistration(TRUE);
              $validation_result->getCacheableMetadata()->applyTo($elements);

              if ($validation_result->isValid()) {
                $entity_type_id = $host_entity->getEntityTypeId();
                $url = Url::fromRoute("entity.$entity_type_id.registration.register", [
                  $entity_type_id => $host_entity->id(),
                ]);
                // Add extra route parameter for the parent product if needed.
                if ($entity_type_id == 'commerce_product_variation') {
                  $parameters = $url->getRouteParameters();
                  /** @var \Drupal\commerce_product\Entity\ProductVariationInterface $entity */
                  $parameters['commerce_product'] = $entity->getProduct()->id();
                  $url->setRouteParameters($parameters);
                }
                $label = $this->getSetting('label') ?: $registration_type->label();
                $elements[] = [
                  '#markup' => Link::fromTextAndUrl($label, $url)->toString(),
                ];
              }
              elseif ($this->getSetting('show_reason')) {
                $elements[] = [
                  '#markup' => $validation_result->getReason(),
                ];
              }
            }

            // Add the access result to cacheability.
            $cacheability = CacheableMetadata::createFromRenderArray($elements);
            $cacheability->addCacheableDependency($access_result);
            $cacheability->applyTo($elements);
          }
        }
      }
    }
    return $elements;
  }

}
